/*
    Copyright (C) 2005-2009  Erik van Pienbroek

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "gui.h"
#include "config_gui.h"
#include "nntpgrab_automation.h"
#ifdef HAVE_LIBNOTIFY
#include <libnotify/notify.h>
#endif

static NNTPGrabAutoImport *auto_import = NULL;

static void
on_nzb_imported(NNTPGrabAutoImport *auto_import, const char *filename, const char *collection_name, gpointer data)
{
#ifdef HAVE_LIBNOTIFY
    NotifyNotification *notification;
    char *msg;

    msg = g_strdup_printf(_("The file '%s'\nwas successfully imported with the collection name '%s'"), filename, collection_name);
    notification = notify_notification_new(_("NZB File Imported"), msg, NULL, NULL);
    g_free(msg);
    notify_notification_show(notification, NULL);
#endif
}

void
enable_auto_import(void)
{
#if GLIB_CHECK_VERSION(2,16,0)
    char *errmsg = NULL;
    ConfigGUIOpts opts = config_gui_get_opts();

    if (!nntpgrab_automation_enable_auto_import(auto_import, opts.auto_import_directory, &errmsg)) {
        GtkWidget *windowMain = nntpgrab_gui_base_get_widget("windowMain");

        nntpgrab_gui_base_dialog_show(windowMain, errmsg, GTK_MESSAGE_INFO, GTK_BUTTONS_OK);

        g_free(errmsg);
    }
#endif
}

void
disable_auto_import(void)
{
#if GLIB_CHECK_VERSION(2,16,0)
    nntpgrab_automation_disable_auto_import(auto_import);
#endif
}

void
auto_import_initialize(void)
{
#if GLIB_CHECK_VERSION(2,16,0)
    ConfigGUIOpts opts = config_gui_get_opts();

    auto_import = nntpgrab_automation_auto_import_init(glue);
    ng_signal_connect(auto_import, "nzb_imported", NG_CALLBACK(on_nzb_imported), NULL);

    if (opts.enable_auto_import) {
        enable_auto_import();
    }
#endif
}

