/*
    Copyright (C) 2005-2009  Erik van Pienbroek

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef _CONFIGURATION_H_
#define _CONFIGURATION_H_

#include "nntpgrab.h"

typedef struct Configuration Configuration;

#define CONFIGURATION_TYPE_OBJECT       (configuration_get_type ())
#define CONFIGURATION(object)           (G_TYPE_CHECK_INSTANCE_CAST ((object), CONFIGURATION_TYPE_OBJECT, Configuration))
#define CONFIGURATION_CLASS(klass)      (G_TYPE_CHECK_CLASS_CAST ((klass), CONFIGURATION_TYPE_OBJECT, ConfigurationClass))
#define IS_CONFIGURATION(object)        (G_TYPE_CHECK_INSTANCE_TYPE ((object), CONFIGURATION_TYPE_OBJECT))
#define IS_CONFIGURATION_CLASS(klass)   (G_TYPE_CHECK_CLASS_TYPE ((klass), CONFIGURATION_TYPE_OBJECT))
#define CONFIGURATION_GET_CLASS(obj)    (G_TYPE_INSTANCE_GET_CLASS ((obj), CONFIGURATION_TYPE_OBJECT, ConfigurationClass))

Configuration   *configuration_new(char **errmsg, char **warnings);
void             configuration_destroy(Configuration *obj);

GList           *configuration_get_avail_servers(Configuration *obj);
void             configuration_free_avail_servers(Configuration *obj, GList *servers);
ConfigServer    *configuration_get_server_info(Configuration *obj, const char *servername);
gboolean         configuration_add_server(Configuration *obj, ConfigServer new_server, char **errmsg);
gboolean         configuration_del_server(Configuration *obj, const char *servername, char **errmsg);
gboolean         configuration_edit_server(Configuration *obj, const char *servername, ConfigServer new_server, char **errmsg);
ConfigGroupInfo  configuration_get_newsgroup_info(Configuration *obj, const char *servername, const char *newsgroup);
void             configuration_set_newsgroup_info(Configuration *obj, const char *servername, const char *newsgroup, ConfigGroupInfo info);
ConfigOpts       configuration_get_opts(Configuration *obj);
void             configuration_set_opts(Configuration *obj, ConfigOpts opts);
gboolean         configuration_load(Configuration *config, char **errmsg);
gboolean         configuration_save(Configuration *config, char **errmsg);
gboolean         configuration_get_folder_listing(Configuration *obj, const char *parent, GList **folders);
void             configuration_free_folder_listing(Configuration *obj, GList *folders);

#endif /* _CONFIGURATION_H_ */
