/*
    Copyright (C) 2005-2009  Erik van Pienbroek

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef _NNTPGRAB_PLUGIN_UNPACK_H_
#define _NNTPGRAB_PLUGIN_UNPACK_H_

#include "nntpgrab_plugin_base.h"

// These are all the functions that the plugin can call to communicate with the NNTPGrab core
typedef struct _plugin_unpack_imported_funcs {
    int version;

    void (*unpack_progress_update) (const char *filename, double progress);
    void (*unpack_message_received) (const char *filename, const char *message);
    void (*unpack_working_archive_changed) (const char *filename, const char *working_archive);
} PluginUnpackImportedFuncs;

// The plugin needs to exported these methods :

/**
 * Initialize the plugin
 *
 * @arg     funcs   A structure containing all the callback functions the plugin can call to notify the NNTPGrab Core
 *
 * @return          TRUE when the initialization was successfull
 */
gboolean     nntpgrab_plugin_unpack_initialize(PluginUnpackImportedFuncs funcs);

/**
 * Return the API version of the plugin. This is used to verify that the plugin is compatible
 * with the interface the NNTPGrab Core expects
 *
 * @return  NNTPGRAB_PLUGIN_API_VERSION
 */
int          nntpgrab_plugin_unpack_get_version(void);

/**
 * Unpack a file and save it in the given directory
 *
 * @arg     filename            Filename of the file which needs to be extracted
 * @arg     target_directory    Directory to save the decode file in
 * @arg     errmsg              If the decode fails, an error message will be put here
 *
 * @return                      The result of the decode process
 */
gboolean     nntpgrab_plugin_unpack_do_unpack(const char *filename, const char *target_directory, char **errmsg);

#endif /* _NNTPGRAB_PLUGIN_UNPACK_H_ */
