/*
    Copyright (C) 2005-2009  Erik van Pienbroek

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <glib-object.h>
#include "nntpgrab_plugin_schedular.h"
#include "schedular_plugin.h"

typedef struct _unpack_file {
    char filename_without_extension[256];
    char filename[256];
} UnpackFile;

void
unpack_thread_func(gpointer data, gpointer user_data)
{
    UnpackData *unpack_data = (UnpackData *) data;
    SchedularPlugin *schedular = SCHEDULAR_PLUGIN(user_data);
    GList *list;
    GList *files = NULL;
    ConfigOpts opts;
    char *collection_name;
    GDir *dir;
    const char *filename;
    char *directory;
    GError *err = NULL;
    static NGRegex *re = NULL;

    if (!re) {
        re = imported_funcs_regex_compile(".*\\.part(\\d+)\\.rar$");
    }

g_print("Unpack thread triggered!\n");

    // Do we still need to unpack?
    opts = imported_funcs_config_get_opts();
    g_static_mutex_lock(&schedular->mutex);
    if (schedular->abort_flag || !opts.enable_auto_unpack) {
        unpack_data->collection->now_unpacking = FALSE;
        g_static_mutex_unlock(&schedular->mutex);

        imported_funcs_collection_unref(unpack_data->collection);

        g_slice_free(UnpackData, unpack_data);
        return;
    }
    g_static_mutex_unlock(&schedular->mutex);

    g_static_mutex_lock(&unpack_data->collection->mutex);

    request_par2_and_unpack_lock();

    collection_name = g_strdup(unpack_data->collection->collection_name);

    directory = g_build_filename(opts.download_directory, collection_name, NULL);
    dir = g_dir_open(directory, 0, &err);
    if (!dir) {
        g_warning("Unable to open directory '%s' for unpacking (%s)", directory, err->message);
        g_error_free(err);
        g_free(directory);
        unpack_data->collection->now_unpacking = FALSE;

        test_if_all_downloads_are_completed(schedular, unpack_data->poolPar2verify, unpack_data->poolPar2repair, unpack_data->poolUnpack);

        imported_funcs_collection_unref(unpack_data->collection);
        g_slice_free(UnpackData, unpack_data);
    }

    // Find out all the unique archives in this directory
    while ((filename = g_dir_read_name(dir))) {
        // Is this a file in the notation '.partXXX.rar' ?
        char *filename_lowercase = g_ascii_strdown(filename, -1);
        const char **matches;

g_print(__FILE__ ":%i filename = %s\n", __LINE__, filename_lowercase);

        if ((matches = imported_funcs_regex_match(re, filename_lowercase))) {
            // Is this .part001.rar ?
g_print(__FILE__ ":%i matches[1] = %s\n", __LINE__, matches[1]);
            if (matches[0] && matches[1] && atoi(matches[1]) == 1) {
                UnpackFile *file = g_slice_new0(UnpackFile);

                strncpy(file->filename, filename, sizeof(file->filename) - 1);

                files = g_list_append(files, file);
            }

            imported_funcs_regex_matches_free(matches);
        } else if (!strcmp(filename_lowercase + strlen(filename_lowercase) - 4, ".rar")) {
            // A .rar file detected which isn't part of a .partXX.rar set
            UnpackFile *file = g_slice_new0(UnpackFile);

            strncpy(file->filename, filename, sizeof(file->filename) - 1);

            files = g_list_append(files, file);
        } else if (!strcmp(filename_lowercase + strlen(filename_lowercase) - 4, ".zip")) {
            // A .zip file detected
            UnpackFile *file = g_slice_new0(UnpackFile);

            strncpy(file->filename, filename, sizeof(file->filename) - 1);

            files = g_list_append(files, file);
        } else {
g_print(__FILE__ ":%i file ignored\n\n", __LINE__);
        }

        g_free(filename_lowercase);
    }
    g_dir_close(dir);

    g_static_mutex_unlock(&unpack_data->collection->mutex);

    // Perform the unpack
    list = files;
    while (list) {
        UnpackFile *file = (UnpackFile*) list->data;

g_print(__FILE__ ":%i Now unpacking '%s'\n", __LINE__, file->filename);

        imported_funcs_perform_unpack(collection_name, file->filename);

        g_slice_free(UnpackFile, file);
        list = g_list_next(list);
    }
    g_list_free(files);

    g_free(collection_name);

    g_static_mutex_lock(&unpack_data->collection->mutex);
    unpack_data->collection->now_unpacking = FALSE;
    g_static_mutex_unlock(&unpack_data->collection->mutex);

    release_par2_and_unpack_lock();

    test_if_all_downloads_are_completed(schedular, unpack_data->poolPar2verify, unpack_data->poolPar2repair, unpack_data->poolUnpack);

    imported_funcs_collection_unref(unpack_data->collection);

    g_slice_free(UnpackData, unpack_data);
}
